# functions for converting notebooks to learn-style markdown challenges

import pandas as pd
from tabulate import tabulate
from .challenge import *

def make_coding(nb, uuid):
    """Make a learn coding challenge from a jupyter notebook

    Parameters
    ----------
    nb : a jupyter notebook json string
    uuid : unique universal identifier for the challenge

    """
    title = get_source(nb['cells'], 'title')
    title = title.strip('#').strip()

    question = get_source(nb['cells'], tag='question')
    question = to_mathjax(question)

    placeholder = get_source(nb['cells'], tag='placeholder')
    placeholder = strip_magic(placeholder)
    placeholder = strip_solution(placeholder)

    tests = get_source(nb['cells'], tag='tests')
    tests = strip_magic(tests)

    explanation = get_source(nb['cells'], tag='explanation')
    explanation = to_mathjax(explanation)
    solution = get_solution(tests)
    lines = explanation.split('\n')
    lines.append('')
    lines.extend(solution)
    explanation = '\n'.join(lines)

    challenge = \
f"""### !challenge

* type: code-snippet
* language: python3.6
* id: {uuid}
* title: {title}

### !question
{question}
### !end-question

### !placeholder

```python
{placeholder}
```
### !end-placeholder

### !tests
```python
{tests}
```
### !end-tests

### !explanation
{explanation}
### !end-explanation

### !end-challenge"""

    video = get_source(nb['cells'], tag='video')
    if not video:
        return challenge

    lines = challenge.split('\n')
    lines.append('## Worked Solution')
    lines.append('')
    lines.append(video)
    challenge = '\n'.join(lines)
    return challenge

def make_multiple_choice(nb, uuid):
    """Make a learn multiple challenge from a jupyte notebook

    Parameters
    ----------
    nb : a jupyter notebook json string
    uuid : unique universal identifier for the challenge

    """
    title = get_source(nb['cells'], 'title')
    title = title.strip('#').strip()

    question = get_source(nb['cells'], tag='question')
    question = to_mathjax(question)

    options = get_source(nb['cells'], tag='options')
    options = to_mathjax(options)

    answer = get_source(nb['cells'], tag='answer')
    answer = to_mathjax(answer)

    explanation = get_source(nb['cells'], tag='explanation')
    explanation = to_mathjax(explanation)

    return \
f"""### !challenge

* type: multiple-choice
* id: {uuid}
* title: {title}

### !question
{question}
### !end-question

### !options
{options}
### !end-options

### !answer
{answer}
### !end-answer

### !explanation
{explanation}
### !end-explanation

### !end-challenge"""

def make_paragraph(nb, uuid):
    """Make a paragraph challenge from a jupyter notebook

    Parameters
    ----------
    nb : a jupyter notebook json string
    uuid : unique universal identifier for the challenge

    """

    title = get_source(nb['cells'], 'title')
    title = title.strip('#').strip()

    question = get_source(nb['cells'], tag='question')
    question = to_mathjax(question)

    placeholder = get_source(nb['cells'], tag='placeholder')
    placeholder = to_mathjax(placeholder)

    explanation = get_source(nb['cells'], tag='explanation')
    explanation = to_mathjax(explanation)

    return \
f"""### !challenge

* type: paragraph
* id: {uuid}
* title: {title}

### !question
{question}
### !end-question

### !placeholder
{placeholder}
### !end-placeholder

### !explanation
{explanation}
### !end-explanation

### !end-challenge"""

def make_number(nb, uuid, decimal=5):
    """Make a number challenge from a jupyter notebook

    Parameters
    ----------
    nb : a jupyter notebook json string
    uuid : unique universal identifier for the challenge

    """

    title = get_source(nb['cells'], 'title')
    title = title.strip('#').strip()

    question = get_source(nb['cells'], tag='question')
    question = to_mathjax(question)

    placeholder = get_source(nb['cells'], tag='placeholder')
    placeholder = to_mathjax(placeholder)

    answer = get_source(nb['cells'], tag='answer')
    answer = to_mathjax(answer)

    explanation = get_source(nb['cells'], tag='explanation')
    explanation = to_mathjax(explanation)

    return \
f"""### !challenge

* type: number
* id: {uuid}
* title: {title}
* decimal: {decimal}

### !question
{question}
### !end-question

### !placeholder
{placeholder}
### !end-placeholder

### !answer
{answer}
### !end-answer

### !explanation
{explanation}
### !end-explanation

### !end-challenge"""

def make_raw(nb, uuid):
    """Convert notebook to markdown

    Parameters
    ----------
    nb : a jupyter notebook json string

    Format code nice.

    """
    lines = []
    for cell in nb['cells']:
        if cell['cell_type'] == 'markdown':
            lines += cell['source']
        elif cell['cell_type'] == 'code':
            lines += ['```python']
            lines += cell['source']
            lines += ['```']

    markdown = ''.join(lines)
    markdown = to_mathjax(markdown)
    return markdown



import sys
from nbconvert import Exporter
from .challenges import *


CHALLENGES = {'coding': make_coding,
              'multiple_choice': make_multiple_choice,
              'paragraph': make_paragraph,
              'number': make_number,
              'raw': make_raw,
}


class LearnExporter(Exporter):
    output_mimetype = 'application/newformat'

    def _file_extension_default(self):
        return '.md'

    def from_notebook_node(self, nb, resources=None):
        import uuid
        nb_fname = sys.argv[-1]
        challenge_name, challenge_type, md, ipynb = nb_fname.split('.')
        make_challenge = CHALLENGES[challenge_type]
        uuid = uuid.uuid4()
        markdown = make_challenge(nb, str(uuid))

        return markdown, resources



# helper functions for processing learn challenges written in notebooks

import numpy as np
import pandas as pd

def get_solution(tests):
    """Parse out the solution from a coding challenge test class

    Parameters
    ----------
    test : a code cell containing a test class

    There must be a `solution()` method

    """
    lines = tests.split('\n')
    i = 0
    while True:
        line = lines[i]
        if not 'def solution' in line:
            i += 1
            continue
        else:
            break
            
    i += 1
    solution = []
    while True:
        line = lines[i]
        if not line.strip():
            break

        solution.append(line)
        i += 1

    solution = ['```python'] + solution[:-1] + ['```']
    return solution

def strip_magic(s):
    """Strip magic and comments from the beginning of a code cell

    Parameters
    ----------
    s : a code cell beginning with magic and/or comments

    """
    lines = s.split('\n')
    i = 0
    while True:
        line = lines[i]
        if not line or line.startswith('%%') or line.startswith('#'):
            i += 1
            continue
        else:
            break

    lines = lines[i:]
    s = '\n'.join(lines)
    return s

def strip_solution(s):
    """Remove the solution from a code cell

    Parameters
    ----------
    s : a code cell

    This is currently not used anywhere.

    """
    lines = s.split('\n')
    xformed_lines = []
    OUT, IN = True, False
    for line in lines:
        if OUT:
            if line.strip() == '### BEGIN SOLUTION':
                IN, OUT = True, False
            else:
                xformed_lines.append(line)
        else:
            assert IN
            if line.strip() == '### END SOLUTION':
                OUT, IN = True, False
                line = line.replace('### END SOLUTION', 'pass')
                xformed_lines.append(line)
            else:
                continue

    s = '\n'.join(xformed_lines)
    return s

def to_mathjax(text):
    """Convert jupyter-style LaTeX to learn-style MathJax

    Parameters
    ----------
    text : a code cell containing jupyter-style LaTeX

    The only substitutions that are made are
    - \\ -> \\\
    - $...$ -> \\(...\\)

    It's assumed that $$ do not appear inline (i.e. the only place they appear
    is on their own line.

    """
    lines = text.split('\n')
    lines = [line.replace(r'\\', '\\\\\\') for line in lines]
    lines = [line.replace(r'\{', '\\\\{') for line in lines]
    lines = [line.replace(r'\}', '\\\\}') for line in lines]

    xformed_lines = []
    for line in lines:
        if line == '$$':
            xformed_lines.append(line)
            continue

        OUT, IN = True, False
        xformed_line = []
        for c in line:
            if not c == '$':
                xformed_line.append(c)
                continue

            if OUT:
                IN, OUT = True, False
                xformed_line.append(r'\\(')
            elif IN:
                OUT, IN = True, False
                xformed_line.append(r'\\)')

        xformed_line = ''.join(xformed_line)
        xformed_lines.append(xformed_line)

    xformed_text = '\n'.join(xformed_lines)
    return xformed_text

def get_source(cells, tag):
    """Get the contents of a code cell by tag

    Parameters
    ----------
    cells : list of jupyter notebook cells
    tag : tag name

    """
    for cell in cells:
        metadata = cell['metadata']
        if 'tags' not in metadata:
            continue # no tags
        
        if tag in metadata['tags']:
            lines = cell['source'] 
            source = ''.join(lines)
            return source



from setuptools import setup, find_packages
from codecs import open
from os import path

__version__ = '0.0.2'

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# get the dependencies and installs
with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if x.startswith('git+')]

setup(name='julpyter',
      version=__version__,
      description='A python package which converts a jupyter notebook to a Galvanize Learn challenge',
      long_description=long_description,
      url='https://github.com/embanner/julpyter',
      download_url='https://github.com/embanner/julpyter/tarball/' + __version__,
      license='BSD',
      classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
      ],
      keywords='',
      packages=find_packages(exclude=['docs', 'tests*']),
      include_package_data=True,
      author='Edward Banner',
      install_requires=install_requires,
      dependency_links=dependency_links,
      author_email='edward.banner@galvanize.com',
      entry_points={
        'nbconvert.exporters': ['learn = julpyter:LearnExporter'],
      },
)
